Package: BayesFM
Title: Bayesian Inference for Factor Modeling
Type: Package
Version: 0.1.7
Authors@R: person(
    given = "Rémi", 
    family = "Piatek", 
    role = c("aut", "cre"),
    email = "remi.piatek@gmail.com",
    comment = c(ORCID = "0000-0002-3474-1304"))
Description: Collection of procedures to perform Bayesian analysis on a variety
    of factor models. Currently, it includes: "Bayesian Exploratory Factor 
    Analysis" (befa) from G. Conti, S. Frühwirth-Schnatter, J.J. Heckman, 
    R. Piatek (2014) <doi:10.1016/j.jeconom.2014.06.008>, an approach to 
    dedicated factor analysis with stochastic search on the structure of the 
    factor loading matrix. The number of latent factors, as well as the 
    allocation of the manifest variables to the factors, are not fixed a priori 
    but determined during MCMC sampling.
Depends: R (>= 3.0.0)
Imports: checkmate (>= 1.8.0), coda, ggplot2 (>= 2.1.0), gridExtra,
        plyr (>= 1.8.0)
SystemRequirements: gfortran (>= 4.6.3)
License: GPL-3
NeedsCompilation: yes
Encoding: UTF-8
RoxygenNote: 7.3.1
Packaged: 2024-06-09 07:16:09 UTC; remek
Author: Rémi Piatek [aut, cre] (<https://orcid.org/0000-0002-3474-1304>)
Maintainer: Rémi Piatek <remi.piatek@gmail.com>
Repository: CRAN
Date/Publication: 2024-06-13 21:50:16 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-27 23:09:06 UTC; unix
Archs: BayesFM.so.dSYM
