# Generated by roxygen2: do not edit by hand

S3method(plot,befa)
S3method(plot,simul.R.prior)
S3method(plot,simul.nfac.prior)
S3method(print,befa)
S3method(print,simul.R.prior)
S3method(print,simul.nfac.prior)
S3method(print,summary.befa)
S3method(print,summary.simul.R.prior)
S3method(print,summary.simul.nfac.prior)
S3method(summary,befa)
S3method(summary,simul.R.prior)
S3method(summary,simul.nfac.prior)
export(befa)
export(post.column.switch)
export(post.sign.switch)
export(simul.R.prior)
export(simul.dedic.facmod)
export(simul.nfac.prior)
import(checkmate)
importFrom(coda,HPDinterval)
importFrom(coda,as.mcmc)
importFrom(ggplot2,aes_string)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_bar)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_rug)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_build)
importFrom(ggplot2,ggplot_gtable)
importFrom(ggplot2,guide_colorbar)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_gradient2)
importFrom(ggplot2,theme)
importFrom(ggplot2,unit)
importFrom(gridExtra,arrangeGrob)
importFrom(gridExtra,grid.arrange)
importFrom(plyr,count)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov2cor)
importFrom(stats,median)
importFrom(stats,quantile)
importFrom(stats,rWishart)
importFrom(stats,rgamma)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,terms)
useDynLib(BayesFM, .registration = TRUE, .fixes = "F_")
