
BayesFM Package

Installation notes

- This package requires a Fortran compiler and a C compiler
- GNU Fortran version 4.6.3 or later is recommended
- G95 and earlier versions of GNU Fortran may not work because
  of unsupported Fortran 2003 features used in this package

Windows users:
- It is recommended to install Rtools to compile this package from source,
  see R for Windows FAQ https://cran.r-project.org/bin/windows/base/rw-FAQ.html
