module global

  use, intrinsic :: iso_fortran_env
  implicit none

  ! Fortran kinds for single and double precisions
  integer, parameter :: i4 = INT32
  integer, parameter :: i8 = INT64
  integer, parameter :: r4 = REAL32
  integer, parameter :: r8 = REAL64

end module global

